@@ -3,6 +3,7 @@ import json
 import pickle
 from collections import namedtuple
 from contextlib import suppress
+from copy import deepcopy
 
 import numpy as np
 
@@ -91,8 +92,13 @@ class Dict2Data:
                 result.update({k:Dict2Data.to_dict(v)})
             else:
                 result.update({k:v})
-        return result
-    def to_json(self,outfile=None,indent=1):
+        return deepcopy(result) # prevent accidental changes in numpy arrays
+    
+    def copy(self):
+        "Copy of self to avoid changes during inplace operations on numpy arrays."
+        return self.__class__(self.to_dict()) # make a copy of self through dictionary, otherwise it does not work
+    
+    def to_json(self,outfile:str = None,indent:int = 1):
         """Dumps a `Dict2Data` object (root or nested level) to json.
         - **Parameters**
             - outfile : Default is None and returns string. If given, writes to file.
@@ -100,7 +106,7 @@ class Dict2Data:
         """
         return dump(self,dump_to='json',outfile=outfile,indent=indent)
 
-    def to_pickle(self,outfile=None):
+    def to_pickle(self,outfile:str=None):
         """Dumps a `Dict2Data` or subclass object (root or nested level) to pickle.
         - **Parameters**
             - outfile : Default is None and returns string. If given, writes to file.
@@ -293,11 +299,10 @@ class PoscarData(Dict2Data):
         all_dist = all_dist[all_dist > 0] # Remove 0 distances
         return np.min(all_dist)
     
-    def write(self,sd_list = None, outfile = None, overwrite = False):
-        """Writes the poscar data to a file.
-        """
+    def write(self, outfile = None, overwrite = False):
+        "Writes the poscar data to a file."
         from .sio import write_poscar # To avoid circular import
-        return write_poscar(self,sd_list = sd_list,outfile = outfile,overwrite = overwrite)
+        return write_poscar(self,outfile = outfile,overwrite = overwrite)
         
 
 class GridData(Dict2Data):
@@ -311,19 +316,16 @@ class GridData(Dict2Data):
     @property
     def coords(self):
         "Returns coordinates of the grid points in shape (3,Nx, Ny,Nz). (x,y,z) = i/Nx*a + j/Ny*b + k/Nz*c, where (a,b,c) is the lattice vector. and i,j,k are the grid indices. as 0-Nx-1, 0-Ny-1, 0-Nz-1."
-        for a in ['e','m','m_x','m_y','m_z']:
-            if hasattr(self,a):
-                shape = getattr(self,a).shape
-                Nx, Ny, Nz = shape
-                ix,iy,iz = np.indices(shape)
-                a1,a2,a3 = self.poscar.basis
-                return np.array([
-                    ix*a1[0]/Nx + iy*a2[0]/Ny + iz*a3[0]/Nz,
-                    ix*a1[1]/Nx + iy*a2[1]/Ny + iz*a3[1]/Nz,
-                    ix*a1[2]/Nx + iy*a2[2]/Ny + iz*a3[2]/Nz
-                ])
+        shape = self.values.shape
+        Nx, Ny, Nz = shape
+        ix,iy,iz = np.indices(shape)
+        a1,a2,a3 = self.poscar.basis
+        return np.array([
+            ix*a1[0]/Nx + iy*a2[0]/Ny + iz*a3[0]/Nz,
+            ix*a1[1]/Nx + iy*a2[1]/Ny + iz*a3[1]/Nz,
+            ix*a1[2]/Nx + iy*a2[2]/Ny + iz*a3[2]/Nz
+        ])
             
-        raise AttributeError("Coordinates not found. Please check that the grid data is present from any of ['e','m','m_x','m_y','m_z']")
 
 class OutcarData(Dict2Data):
     _req_keys = ('site_pot','ion_pot','basis')
